--[[ ArenaLive Spectator Handler: CooldownTracker
Created by: Vadrak
Creation Date: 07.08.2013
Last Update: "
Description: This handler is used to track cooldowns.
]]--
local addonName, L  = ...;

-- Local version is said to be faster.
local ArenaLiveCore = ArenaLiveCore;

-- Set up a new handler.
local CooldownTracker = ArenaLiveCore:AddHandler("CooldownTracker", "EventCore");

-- Get Cooldown Handler:
local Cooldown = ArenaLiveCore:GetHandler("Cooldown");

-- Register the handler for all needed events.
CooldownTracker:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");

-- List of all cooldown trackers in order to hand over events to them:
local cooldownTrackers = {};

local throttleUpdate = 0.2;
local function OnUpdate(self, elapsed)
	
	self.elapsed = self.elapsed - elapsed;
	
	if ( self.elapsed <= 0 ) then
		self.elapsed = throttleUpdate;
		local theTime = GetTime();
		
		for key, cooldownInfo in ipairs(self.cooldowns) do
			if ( cooldownInfo["expires"] <= theTime ) then
				table.remove(self.cooldowns, key);
				self:Update();
			end
		end
	end

end

local function Update(self)
	
	local maxIcons = ArenaLiveCore:GetDBEntry(self.addonName, "CooldownTracker/MaxIcons"); 
	
	-- First sort all cooldowns by time:
	CooldownTracker:SortCooldowns(self);
	
	-- Then start to show them:
	for i = 1, maxIcons do
		local icon = self["icon"..i];
		
		if ( self.cooldowns[i] ) then
			if ( not icon ) then
				icon = CooldownTracker:CreateIcon(self);
			end
			local spellID = self.cooldowns[i]["spellID"]
			local duration = ArenaLiveCore.spellDB.Cooldowns[spellID];
			local startTime = self.cooldowns[i]["expires"] - duration; 
			
			icon.texture:SetTexture(self.cooldowns[i]["texture"]);
			icon.cooldown:Set(startTime, duration);
			icon:Show();
		else
			if ( icon ) then
				icon:Hide();
			end
		end
	end
end

local function Reset(self)

	table.wipe(self.cooldowns);
	table.wipe(self.trackedUnits);
	
	for i = 1, self.numIcons do
		local icon = self["icon"..i];
		
		if ( icon:IsShown() ) then
			icon:Hide();
			icon.texture:SetTexture();
			icon.cooldown:Reset();
		end
	end

end

function CooldownTracker:AddFrame(tracker, addonName, updateTrackedUnitsFunc, iconTemplate)

	-- Create a tables to store cooldown information:
	tracker.cooldowns = {};
	tracker.trackedUnits = {};
	
	-- Set base variables:
	tracker.elapsed = 0;
	tracker.numIcons = 0;
	tracker.addonName = addonName;
	tracker.iconTemplate = iconTemplate;
	
	-- Set functions
	tracker.OnUpdate = OnUpdate;
	tracker.Update = Update;
	tracker.Reset = Reset;
	tracker.UpdateTrackedUnits = updateTrackedUnitsFunc;
	
	-- Set OnUpdate Script:
	tracker:SetScript("OnUpdate", tracker.OnUpdate);
	
	-- Update Once at start up:
	tracker:UpdateTrackedUnits();
	
	-- Add to cooldown tracker list:
	cooldownTrackers[tracker] = true;
end

function CooldownTracker:AddCooldown(tracker, unit, spellID)
	if ( tracker:IsVisible() ) then
		local guid = UnitGUID(unit);
		local _, _, icon = GetSpellInfo(spellID);
		local expires = ArenaLiveCore.spellDB.Cooldowns[spellID];
		
		if ( icon and expires ) then
			expires = expires + GetTime();
			
			-- Check if there is already a spell cooldown of this spell from the same player:
			local existingCooldown = false;
			for key, cooldownInfo in ipairs(tracker.cooldowns) do
				if ( spellID == cooldownInfo["spellID"] and guid == cooldownInfo["guid"] ) then
					existingCooldown = key;
				end
			end
			
			if ( existingCooldown ) then
				tracker.cooldowns[existingCooldown]["expires"] = expires;
			else
				table.insert(tracker.cooldowns, { ["expires"] = expires, ["spellID"] = spellID, ["guid"] = guid, ["texture"] = icon, } );
			end
			
			tracker:Update();
		end
	end
end

function CooldownTracker:SortCooldowns(tracker)
	table.sort(tracker.cooldowns, function (a, b) return a.expires > b.expires end);
end

function CooldownTracker:CreateIcon(tracker)
	
	local prefix = tracker:GetName();
	local numIcon = tracker.numIcons + 1;
	local iconName = prefix.."Icon"..numIcon;
	
	-- Create Frame:
	local icon = CreateFrame("Frame", iconName, tracker, tracker.iconTemplate);
	
	-- Set Reference in tracker:
	tracker["icon"..numIcon] = icon;
	
	-- Set Anchor:
	local xOffset = ArenaLiveCore:GetDBEntry(tracker.addonName, "CooldownTracker/Icon/XOffset"); 
	local yOffset = ArenaLiveCore:GetDBEntry(tracker.addonName, "CooldownTracker/Icon/YOffset"); 
	if ( numIcon == 1 ) then
		icon:SetPoint("BOTTOM", tracker, "BOTTOM", xOffset, yOffset);
	else
		icon:SetPoint("BOTTOM", tracker["icon"..numIcon-1], "TOP", xOffset, yOffset);
	end
	
	-- Set up cooldown if there is one:
	local cooldown = _G[prefix.."Icon"..numIcon.."Cooldown"];
	if ( cooldown ) then
		Cooldown:AddFrame(cooldown, _G[prefix.."Icon"..numIcon.."CooldownText"], icon, true);
	end
	
	-- Set Up Texture:
	local texture = _G[prefix.."Icon"..numIcon.."Texture"];
	icon.texture = texture;

	tracker.numIcons = tracker.numIcons + 1
	
	return icon;
end

function CooldownTracker:OnEvent(event, ...)
		
	if ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
		local unit = ...;
		local spellID = select(5, ...);
		if ( ArenaLiveCore.spellDB.Cooldowns[spellID] ) then
			for tracker, value in pairs(cooldownTrackers) do
				if ( tracker and value and tracker.trackedUnits[unit] ) then
					CooldownTracker:AddCooldown(tracker, unit, spellID);
				end
			end
		end
	end
end