--[[ ArenaLive Spectator Handler: UnitTracker
Created by: Vadrak
Creation Date: 07.08.2013
Last Update: "
Description: This handler is used to track the current names of all unitIDs that are relevant to show characters in the spectator UI's frames, i.e. all long term IDs like player, party, raid and arena IDs.
]]--
local addonName, L  = ...;

-- Local version is said to be faster.
local ArenaLiveCore = ArenaLiveCore;

-- Set up a new handler.
local UnitTracker = ArenaLiveCore:AddHandler("UnitTracker", "EventCore");

-- Register the handler for all needed events.
UnitTracker:RegisterEvent("GROUP_ROSTER_UPDATE");
UnitTracker:RegisterEvent("ARENA_OPPONENT_UPDATE");
UnitTracker:RegisterEvent("PLAYER_ENTERING_WORLD");

-- Create tables to store unit names and IDs sorted both ways to prevent the need to iterate through the tables via for loops.
ArenaLiveSpectator.UnitsByNameTable = {};
ArenaLiveSpectator.NamesByUnitTable = {};

-- Create Variables to store the maximal number of players of a particular unitID group.
local MAX_PARTY_MEMBERS = 4;
local MAX_RAID_MEMBERS = 40;
local MAX_ARENA_ENEMIES = 5;

function UnitTracker:UpdateUnit(unit)

	if ( not unit ) then
		return;
	end
	
	local name, oldUnit, oldName, updateUnitTable, updateNameTable;
	
	
	if ( UnitExists(unit) ) then
	
		name = ArenaLiveSpectator:GetUnitName(unit);
		oldUnit = ArenaLiveSpectator.UnitsByNameTable[name];
		oldName = ArenaLiveSpectator.NamesByUnitTable[unit];
		
		if ( oldName ) then
			if ( name ~= oldName ) then
				updateUnitTable = true;
				
				-- Reset old name entry already at this point if needed:
				if ( ArenaLiveSpectator.UnitsByNameTable[oldName] == unit ) then
					ArenaLiveSpectator.UnitsByNameTable[oldName] = nil;
				end					
			end
		else
			updateUnitTable = true;
		end	
		
		if ( oldUnit ) then
			local currentOldUnitName = ArenaLiveSpectator:GetUnitName(oldUnit);
			if ( name ~= currentOldUnitName or UnitTracker:UnitIsBetterThanUnit(unit, oldUnit) ) then
				updateNameTable = true;
			end
		else
			updateNameTable = true;
		end
		
		if ( updateUnitTable ) then
			ArenaLiveSpectator.NamesByUnitTable[unit] = name;
		end		
		
		if ( updateNameTable ) then
			ArenaLiveSpectator.UnitsByNameTable[name] = unit;
		end	

	else
		-- Reset Unit if unit doesn't exist:
		oldName = ArenaLiveSpectator.NamesByUnitTable[unit];
		ArenaLiveSpectator.NamesByUnitTable[unit] = nil;
		
		if ( oldName and ArenaLiveSpectator.UnitsByNameTable[oldName] == unit ) then
			ArenaLiveSpectator.UnitsByNameTable[oldName] = nil;
		end
	end
	
end

function UnitTracker:UpdatePlayer()
	local unit = "player";
	local name = ArenaLiveSpectator:GetUnitName(unit);

	ArenaLiveSpectator.UnitsByNameTable[name] = unit;
	ArenaLiveSpectator.NamesByUnitTable[unit] = name;
	ArenaLiveSpectator:OnEvent("ALSPEC_AVAILABLE_UNITS_UPDATE");
end

function UnitTracker:UpdatePartyOrRaid()
	local unit, oldUnit, name;
	for i = 1, MAX_RAID_MEMBERS do
		if ( i <= MAX_PARTY_MEMBERS ) then
			UnitTracker:UpdateUnit("party"..i);
		end
		UnitTracker:UpdateUnit("raid"..i);
	end
	ArenaLiveSpectator:OnEvent("ALSPEC_AVAILABLE_UNITS_UPDATE");
end

function UnitTracker:UpdateArenaOpponent(unit, state)
	local oldUnit, name;
	
	--[[ INFORMATION ON ARENA_OPPONENT_UPDATE:
		 seen = An enemy gets visible to the player.
		 unseen = An enemy gets invisible to the player or releases his/her Ghost.
		 destroyed = An enemy leaves the Arena.
		 cleared = This fires, after you leave the Arena.
	]]--	
	if ( state == "seen" ) then
		UnitTracker:UpdateUnit(unit);
		ArenaLiveSpectator:OnEvent("ALSPEC_AVAILABLE_UNITS_UPDATE");
	elseif ( state == "destroyed" or state == "cleared" ) then
		-- Maybe a new enemy:
		UnitTracker:UpdateUnit(unit);
		ArenaLiveSpectator:OnEvent("ALSPEC_AVAILABLE_UNITS_UPDATE");
	elseif ( not unit and not state ) then
		for i = 1, MAX_ARENA_ENEMIES do
			UnitTracker:UpdateUnit("arena"..i);
		end
	end
	
end

function UnitTracker:UpdateAll()
	UnitTracker:UpdateArenaOpponent();
	UnitTracker:UpdatePartyOrRaid();
	UnitTracker:UpdatePlayer();
	ArenaLiveSpectator:CheckAllSideFrameUnits();
end

function UnitTracker:OnEvent(event, ...)
	
	if ( event == "GROUP_ROSTER_UPDATE" ) then
		UnitTracker:UpdatePartyOrRaid()
	elseif ( event == "ARENA_OPPONENT_UPDATE" ) then
		local unit, state = ...;
		UnitTracker:UpdateArenaOpponent(unit, state);
	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		UnitTracker:UpdateAll();
	end

end

local unitTypeValues =
	{
		["target"] = -1,
		["arena"] = 1,
		["party"] = 2,
		["raid"] = 3,
		["player"] = 4,
	}
function UnitTracker:UnitIsBetterThanUnit(unit1, unit2)

	if ( not unit1 and not unit2 ) then
		return "error";
	end
	
	local unit1Type = string.match(unit1, "^([a-z]+)[0-9]+$") or unit1;
	local unit2Type = string.match(unit2, "^([a-z]+)[0-9]+$") or unit2;
	local unit1Value = unitTypeValues[unit1Type];
	local unit2Value = unitTypeValues[unit2Type];
	
	if ( not unit1Value ) then
		return false;
	elseif ( not unit2Value ) then
		return true;
	elseif ( unit1Value > unit2Value ) then
		return true;
	elseif ( unit1Value < unit2Value ) then
		return false;
	elseif ( unit1Value == unit2Value ) then
		return false;
	end
	
end

function UnitTracker:PrintTables()
	print ("ArenaLiveSpectator.UnitsByNameTable:")
	for name, unit in pairs(ArenaLiveSpectator.UnitsByNameTable) do
		print("     "..name.." = "..unit);
	end

	print ("ArenaLiveSpectator.NamesByUnitTable:")
	for unit, name in pairs(ArenaLiveSpectator.NamesByUnitTable) do
		print("     "..unit.." = "..name);
	end	
	
end